@@ -208,7 +208,8 @@ on_context_unavailable (GUPnPContextManager    *manager,
                 g_signal_stop_emission_by_name (manager, "context-unavailable");
 
                 priv->filtered = g_list_remove_link (priv->filtered, ctx);
-                g_object_unref (ctx);
+                g_object_unref (ctx->data);
+                g_list_free (ctx);
         }
 
         g_hash_table_remove (priv->contexts, context);
@@ -479,7 +480,7 @@ gupnp_context_manager_dispose (GObject *object)
         g_ptr_array_free (priv->control_points, TRUE);
         g_ptr_array_free (priv->root_devices, TRUE);
 
-        g_list_free (priv->filtered);
+        g_list_free_full (priv->filtered, g_object_unref);
         priv->filtered = NULL;
 
         g_clear_object (&filter);
