@@ -104,6 +104,7 @@ get_description_url_data_free (GetDescriptionURLData *data)
         g_free (data->service_type);
         g_free (data->description_url);
         g_object_unref (data->control_point);
+        g_object_unref (data->cancellable);
 
         g_slice_free (GetDescriptionURLData, data);
 }
@@ -579,6 +580,7 @@ got_description_url (GObject *source,
         GUPnPXMLDoc *doc;
         GUPnPControlPointPrivate *priv;
         GError *error = NULL;
+        gboolean retry = FALSE;
         SoupMessage *message =
                 soup_session_get_async_result_message (SOUP_SESSION (source),
                                                        res);
@@ -590,9 +592,17 @@ got_description_url (GObject *source,
         if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
                 goto out;
 
+        if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT)) {
+                g_clear_error (&error);
+                retry = TRUE;
+        }
+
         // FIXME: This needs better handling
-        if (error != NULL)
-                g_assert_not_reached ();
+        if (error != NULL) {
+                g_warning ("Retrieving the description document failed: %s",
+                           error->message);
+                goto out;
+        }
 
         priv = gupnp_control_point_get_instance_private (data->control_point);
 
@@ -611,7 +621,8 @@ got_description_url (GObject *source,
         }
 
         /* Not cached */
-        if (SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (message))) {
+        if (!retry &&
+            SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (message))) {
                 xmlDoc *xml_doc;
                 gsize length;
                 gconstpointer body_data;
@@ -656,7 +667,9 @@ got_description_url (GObject *source,
                         g_warning (
                                 "Failed to GET %s: %s, retrying in %d seconds",
                                 data->description_url,
-                                soup_message_get_reason_phrase (message),
+                                !retry ? soup_message_get_reason_phrase (
+                                                 message)
+                                       : "Timed out",
                                 data->timeout);
 
                         data->timeout_source = g_timeout_source_new_seconds
@@ -699,6 +712,8 @@ load_description (GUPnPControlPoint *control_point,
         GUPnPXMLDoc *doc;
         GUPnPControlPointPrivate *priv;
 
+        g_debug ("Loading description document %s", description_url);
+
         priv = gupnp_control_point_get_instance_private (control_point);
         doc = g_hash_table_lookup (priv->doc_cache,
                                    description_url);
