@@ -54,147 +54,6 @@ read_out_parameter (const char *arg_name,
 }
 
 
-/* Checks an action response for errors and returns the parsed
- * xmlDoc object. */
-static xmlDoc *
-check_action_response (G_GNUC_UNUSED GUPnPServiceProxy *proxy,
-                       GUPnPServiceProxyAction         *action,
-                       xmlNode                        **params,
-                       GError                         **error)
-{
-        xmlDoc *response;
-        int code;
-
-        if (action->msg == NULL) {
-                g_set_error (error,
-                             GUPNP_SERVER_ERROR,
-                             GUPNP_SERVER_ERROR_INVALID_RESPONSE,
-                             "No message, the action was not sent?");
-
-                return NULL;
-        }
-
-        /* Check for errors */
-        switch (action->msg->status_code) {
-        case SOUP_STATUS_OK:
-        case SOUP_STATUS_INTERNAL_SERVER_ERROR:
-                // This indicates a SOAP error that needs to be evaluated
-                break;
-        default:
-                _gupnp_error_set_server_error (error, action->msg);
-
-                return NULL;
-        }
-
-        /* Parse response */
-        response = xmlRecoverMemory (action->msg->response_body->data,
-                                     action->msg->response_body->length);
-
-        if (!response) {
-                if (action->msg->status_code == SOUP_STATUS_OK) {
-                        g_set_error (error,
-                                     GUPNP_SERVER_ERROR,
-                                     GUPNP_SERVER_ERROR_INVALID_RESPONSE,
-                                     "Could not parse SOAP response");
-                } else {
-                        // INTERNALL_SERVER_ERROR was not a SOAP error but something else
-                        g_set_error_literal
-                                    (error,
-                                     GUPNP_SERVER_ERROR,
-                                     GUPNP_SERVER_ERROR_INTERNAL_SERVER_ERROR,
-                                     action->msg->reason_phrase);
-                }
-
-                return NULL;
-        }
-
-        /* Get parameter list */
-        *params = xml_util_get_element ((xmlNode *) response,
-                                        "Envelope",
-                                        NULL);
-        if (*params != NULL)
-                *params = xml_util_real_node ((*params)->children);
-
-        if (*params != NULL) {
-                if (strcmp ((const char *) (*params)->name, "Header") == 0)
-                        *params = xml_util_real_node ((*params)->next);
-
-                if (*params != NULL)
-                        if (strcmp ((const char *) (*params)->name, "Body") != 0)
-                                *params = NULL;
-        }
-
-        if (*params != NULL)
-                *params = xml_util_real_node ((*params)->children);
-
-        if (*params == NULL) {
-                g_set_error (error,
-                             GUPNP_SERVER_ERROR,
-                             GUPNP_SERVER_ERROR_INVALID_RESPONSE,
-                             "Invalid Envelope");
-
-                xmlFreeDoc (response);
-
-                return NULL;
-        }
-
-        /* Check whether we have a Fault */
-        if (action->msg->status_code == SOUP_STATUS_INTERNAL_SERVER_ERROR) {
-                xmlNode *param;
-                char *desc;
-
-                param = xml_util_get_element (*params,
-                                              "detail",
-                                              "UPnPError",
-                                              NULL);
-
-                if (!param) {
-                        g_set_error (error,
-                                     GUPNP_SERVER_ERROR,
-                                     GUPNP_SERVER_ERROR_INVALID_RESPONSE,
-                                     "Invalid Fault");
-
-                        xmlFreeDoc (response);
-
-                        return NULL;
-                }
-
-                /* Code */
-                code = xml_util_get_child_element_content_int
-                                        (param, "errorCode");
-                if (code == -1) {
-                        g_set_error (error,
-                                     GUPNP_SERVER_ERROR,
-                                     GUPNP_SERVER_ERROR_INVALID_RESPONSE,
-                                     "Invalid Fault");
-
-                        xmlFreeDoc (response);
-
-                        return NULL;
-                }
-
-                /* Description */
-                desc = xml_util_get_child_element_content_glib
-                                        (param, "errorDescription");
-                if (desc == NULL)
-                        desc = g_strdup (action->msg->reason_phrase);
-
-                g_set_error_literal (error,
-                                     GUPNP_CONTROL_ERROR,
-                                     code,
-                                     desc);
-
-                g_free (desc);
-
-                xmlFreeDoc (response);
-
-                return NULL;
-        }
-
-        return response;
-}
-
-
 GUPnPServiceProxyAction *
 gupnp_service_proxy_action_new_internal (const char *action) {
         GUPnPServiceProxyAction *ret;
@@ -209,6 +68,16 @@ gupnp_service_proxy_action_new_internal (const char *action) {
         return ret;
 }
 
+
+/**
+ * gupnp_service_proxy_action_ref:
+ * @action: an action
+ *
+ * Increases reference count of `action`
+ *
+ * Returns: (nullable): @action with an increased reference count
+ * Since: 1.2.0
+ */
 GUPnPServiceProxyAction *
 gupnp_service_proxy_action_ref (GUPnPServiceProxyAction *action)
 {
@@ -220,28 +89,30 @@ gupnp_service_proxy_action_ref (GUPnPServiceProxyAction *action)
 static void
 action_dispose (GUPnPServiceProxyAction *action)
 {
-        if (action->proxy != NULL) {
-                g_object_remove_weak_pointer (G_OBJECT (action->proxy),
-                                (gpointer *)&(action->proxy));
-                gupnp_service_proxy_remove_action (action->proxy, action);
-        }
-
-        if (action->cancellable != NULL && action->cancellable_connection_id != 0) {
-                g_cancellable_disconnect (action->cancellable,
-                                          action->cancellable_connection_id);
-        }
-        g_clear_object (&action->cancellable);
         g_clear_error (&action->error);
         g_clear_object (&action->msg);
+
         if (action->msg_str != NULL) {
                 g_string_free (action->msg_str, TRUE);
                 action->msg_str = NULL;
         }
         g_hash_table_destroy (action->arg_map);
         g_ptr_array_unref (action->args);
+        g_clear_pointer (&action->response, g_bytes_unref);
+        g_clear_pointer (&action->doc, xmlFreeDoc);
+
         g_free (action->name);
 }
 
+/**
+ * gupnp_service_proxy_action_unref:
+ * @action: an action
+ *
+ * Decreases reference count of `action`. If reference count drops to 0,
+ * the action and its contents will be freed.
+ *
+ * Since: 1.2.0
+ */
 void
 gupnp_service_proxy_action_unref (GUPnPServiceProxyAction *action)
 {
@@ -295,8 +166,7 @@ write_footer (GUPnPServiceProxyAction *action)
  * gupnp_service_proxy_action_get_result_list() or
  * gupnp_service_proxy_action_get_result_hash()
  *
- * <informalexample>
- * <programlisting>
+ * ```c
  * GUPnPServiceProxyAction *action =
  *         gupnp_service_proxy_action_new ("GetVolume",
  *                                         // Parameters
@@ -322,8 +192,7 @@ write_footer (GUPnPServiceProxyAction *action)
  * }
  *
  * gupnp_service_proxy_action_unref (action);
- * </programlisting>
- * </informalexample>
+ * ```
  *
  * Returns: A newly created #GUPnPServiceProxyAction
  */
@@ -388,8 +257,7 @@ gupnp_service_proxy_action_new (const char *action,
  * gupnp_service_proxy_action_get_result(),
  * gupnp_service_proxy_action_get_result_list() or
  * gupnp_service_proxy_action_get_result_hash()
- * <informalexample>
- * <programlisting>
+ * ```c
  * GList *in_args = NULL;
  * in_args = g_list_append (in_args, "InstanceID");
  * in_args = g_list_append (in_args, "Unit");
@@ -413,8 +281,7 @@ gupnp_service_proxy_action_new (const char *action,
  * GError *error = NULL;
  * gupnp_service_proxy_call_action_async (proxy, action, NULL, on_action_finished, NULL);
  * gupnp_service_proxy_action_unref (action);
- * </programlisting>
- * </informalexample>
+ * ```
  *
  * Returns: A newly created #GUPnPServiceProxyAction
  */
@@ -483,6 +350,152 @@ gupnp_service_proxy_action_serialize (GUPnPServiceProxyAction *action,
         g_string_insert (action->msg_str, action->header_pos, "\">");
 }
 
+/* Checks an action response for errors and returns the parsed
+ * xmlDoc object. */
+void
+gupnp_service_proxy_action_check_response (GUPnPServiceProxyAction *action)
+{
+        xmlDoc *response;
+        int code;
+
+        if (action->doc != NULL) {
+                return;
+        }
+
+        if (action->error != NULL) {
+                return;
+        }
+
+        if (action->msg == NULL || action->response == NULL) {
+                g_set_error_literal (&action->error,
+                                     GUPNP_SERVER_ERROR,
+                                     GUPNP_SERVER_ERROR_INVALID_RESPONSE,
+                                     "No message, the action was not sent?");
+
+                return;
+        }
+
+        SoupStatus status = soup_message_get_status (action->msg);
+
+        if (status != SOUP_STATUS_OK &&
+            status != SOUP_STATUS_INTERNAL_SERVER_ERROR) {
+                _gupnp_error_set_server_error (&action->error, action->msg);
+                return;
+        }
+
+        /* Parse response */
+        gconstpointer data;
+        gsize length;
+        data = g_bytes_get_data (action->response, &length);
+        response = xmlRecoverMemory (data, length);
+        g_clear_pointer (&action->response, g_bytes_unref);
+
+        if (!response) {
+                if (status == SOUP_STATUS_OK) {
+                        g_set_error_literal (
+                                &action->error,
+                                GUPNP_SERVER_ERROR,
+                                GUPNP_SERVER_ERROR_INVALID_RESPONSE,
+                                "Could not parse SOAP response");
+                } else {
+                        g_set_error_literal (
+                                &action->error,
+                                GUPNP_SERVER_ERROR,
+                                GUPNP_SERVER_ERROR_INTERNAL_SERVER_ERROR,
+                                soup_message_get_reason_phrase (action->msg));
+                }
+
+                return;
+        }
+
+        xmlNodePtr params = NULL;
+        /* Get parameter list */
+        params = xml_util_get_element ((xmlNode *) response, "Envelope", NULL);
+        if (params != NULL)
+                params = xml_util_real_node (params->children);
+
+        if (params != NULL) {
+                if (strcmp ((const char *) params->name, "Header") == 0)
+                        params = xml_util_real_node (params->next);
+
+                if (params != NULL)
+                        if (strcmp ((const char *) params->name, "Body") != 0)
+                                params = NULL;
+        }
+
+        if (params != NULL)
+                params = xml_util_real_node (params->children);
+
+        if (params == NULL) {
+                g_set_error (&action->error,
+                             GUPNP_SERVER_ERROR,
+                             GUPNP_SERVER_ERROR_INVALID_RESPONSE,
+                             "Invalid Envelope");
+
+                xmlFreeDoc (response);
+
+                return;
+        }
+
+        /* Check whether we have a Fault */
+        if (status == SOUP_STATUS_INTERNAL_SERVER_ERROR) {
+                xmlNode *param;
+                char *desc;
+
+                param = xml_util_get_element (params,
+                                              "detail",
+                                              "UPnPError",
+                                              NULL);
+
+                if (!param) {
+                        g_set_error (&action->error,
+                                     GUPNP_SERVER_ERROR,
+                                     GUPNP_SERVER_ERROR_INVALID_RESPONSE,
+                                     "Invalid Fault");
+
+                        xmlFreeDoc (response);
+
+                        return;
+                }
+
+                /* Code */
+                code = xml_util_get_child_element_content_int (param,
+                                                               "errorCode");
+                if (code == -1) {
+                        g_set_error (&action->error,
+                                     GUPNP_SERVER_ERROR,
+                                     GUPNP_SERVER_ERROR_INVALID_RESPONSE,
+                                     "Invalid Fault");
+
+                        xmlFreeDoc (response);
+
+                        return;
+                }
+
+                /* Description */
+                desc = xml_util_get_child_element_content_glib (
+                        param,
+                        "errorDescription");
+                if (desc == NULL)
+                        desc = g_strdup (
+                                soup_message_get_reason_phrase (action->msg));
+
+                g_set_error_literal (&action->error,
+                                     GUPNP_CONTROL_ERROR,
+                                     code,
+                                     desc);
+
+                g_free (desc);
+
+                xmlFreeDoc (response);
+
+                return;
+        }
+
+        action->params = params;
+        action->doc = response;
+}
+
 /**
  * gupnp_service_proxy_action_get_result_list:
  * @action: A #GUPnPServiceProxyAction handle
@@ -495,11 +508,11 @@ gupnp_service_proxy_action_serialize (GUPnPServiceProxyAction *action,
  * @error:(inout)(optional)(nullable): The location where to store any error, or %NULL
  *
  * A variant of gupnp_service_proxy_action_get_result() that takes lists of
- * out-parameter names, types and place-holders for values. The returned list
- * in @out_values must be freed using #g_list_free and each element in it using
- * #g_value_unset and #g_free.
- * <informalexample>
- * <programlisting>
+ * out-parameter names, types and place-holders for values.
+ *
+ * The returned list in @out_values must be freed using `g_list_free` and each element
+ * in it using `g_value_unset` and `g_free`.
+ * ```c
  * void on_action_finished(GObject *object, GAsyncResult *res, gpointer user_data)
  * {
  *     GUPnPServiceProxyAction *action;
@@ -543,8 +556,7 @@ gupnp_service_proxy_action_serialize (GUPnPServiceProxyAction *action,
  *     }
  *     g_list_free (out_values);
  * }
- * </programlisting>
- * </informalexample>
+ *```
  *
  * Return value : %TRUE on success.
  *
@@ -558,8 +570,6 @@ gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
                                             GList                  **out_values,
                                             GError                 **error)
 {
-        xmlDoc *response;
-        xmlNode *params;
         GList *names;
         GList *types;
         GList *out_values_list;
@@ -576,8 +586,8 @@ gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
         }
 
         /* Check response for errors and do initial parsing */
-        response = check_action_response (NULL, action, &params, &action->error);
-        if (response == NULL) {
+        gupnp_service_proxy_action_check_response (action);
+        if (action->error != NULL) {
                 g_propagate_error (error, g_error_copy (action->error));
 
                 return FALSE;
@@ -591,7 +601,7 @@ gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
                 val = g_new0 (GValue, 1);
                 g_value_init (val, (GType) types->data);
 
-                read_out_parameter (names->data, val, params);
+                read_out_parameter (names->data, val, action->params);
 
                 out_values_list = g_list_append (out_values_list, val);
 
@@ -600,9 +610,6 @@ gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
 
         *out_values = out_values_list;
 
-        /* Cleanup */
-        xmlFreeDoc (response);
-
         return TRUE;
 
 }
@@ -620,8 +627,7 @@ gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
  * The @out_hash needs to be pre-initialized with key value pairs denoting the argument
  * to retrieve and an empty #GValue initialized to hold the wanted type with g_value_init().
  *
- * <informalexample>
- * <programlisting>
+ *```c
  * void on_action_finished(GObject *object, GAsyncResult *res, gpointer user_data)
  * {
  *     GUPnPServiceProxyAction *action;
@@ -659,8 +665,7 @@ gupnp_service_proxy_action_get_result_list (GUPnPServiceProxyAction *action,
  *
  *     g_hash_table_unref (out_args);
  * }
- * </programlisting>
- * </informalexample>
+ * ```
  *
  * Return value: %TRUE on success.
  *
@@ -671,9 +676,6 @@ gupnp_service_proxy_action_get_result_hash (GUPnPServiceProxyAction *action,
                                             GHashTable              *hash,
                                             GError                 **error)
 {
-        xmlDoc *response;
-        xmlNode *params;
-
         g_return_val_if_fail (action, FALSE);
 
         /* Check for saved error from begin_action() */
@@ -684,18 +686,17 @@ gupnp_service_proxy_action_get_result_hash (GUPnPServiceProxyAction *action,
         }
 
         /* Check response for errors and do initial parsing */
-        response = check_action_response (NULL, action, &params, &action->error);
-        if (response == NULL) {
+        gupnp_service_proxy_action_check_response (action);
+        if (action->error != NULL) {
                 g_propagate_error (error, g_error_copy (action->error));
 
                 return FALSE;
         }
 
         /* Read arguments */
-        g_hash_table_foreach (hash, (GHFunc) read_out_parameter, params);
-
-        /* Cleanup */
-        xmlFreeDoc (response);
+        g_hash_table_foreach (hash,
+                              (GHFunc) read_out_parameter,
+                              action->params);
 
         return TRUE;
 
@@ -720,41 +721,16 @@ gupnp_service_proxy_action_get_result (GUPnPServiceProxyAction *action,
                                        GError                 **error,
                                        ...)
 {
+        g_return_val_if_fail (action, FALSE);
+
         va_list var_args;
-        gboolean ret;
 
         va_start (var_args, error);
-        ret = gupnp_service_proxy_action_get_result_valist (action,
-                                                            error,
-                                                            var_args);
-        va_end (var_args);
-
-        return ret;
-}
-
-/**
- * gupnp_service_proxy_action_get_result_valist:
- * @action: A #GUPnPServiceProxyAction handle
- * @error: (inout)(optional)(nullable): The location where to store any error, or %NULL
- * @var_args: A va_list of tuples of out parameter name, out parameter type,
- * and out parameter value location. The out parameter values should be
- * freed after use
- *
- * See gupnp_service_proxy_action_get_result().
- *
- * Return value: %TRUE on success.
- **/
-gboolean
-gupnp_service_proxy_action_get_result_valist (GUPnPServiceProxyAction *action,
-                                              GError                 **error,
-                                              va_list                  var_args)
-{
         GHashTable *out_hash;
         va_list var_args_copy;
         gboolean result;
         GError *local_error;
 
-        g_return_val_if_fail (action, FALSE);
 
         out_hash = g_hash_table_new_full (g_str_hash,
                                           g_str_equal,
@@ -774,6 +750,7 @@ gupnp_service_proxy_action_get_result_valist (GUPnPServiceProxyAction *action,
         }
         va_end (var_args_copy);
         g_hash_table_unref (out_hash);
+        va_end (var_args);
 
         return result;
 }
