@@ -7,6 +7,8 @@
  *
  */
 
+#define G_LOG_DOMAIN "gupnp-service-proxy"
+
 #include <config.h>
 #include <libsoup/soup.h>
 #include <gobject/gvaluecollector.h>
@@ -622,8 +624,8 @@ gupnp_service_proxy_add_notify_full (GUPnPServiceProxy              *proxy,
  *
  * Get a notification for anything that happens on the peer.
  *
- * @value in @callback will be of type #G_TYPE_POINTER and contain the pre-parsed
- * #xmlDoc. Do NOT free or modify this document.
+ * @value in @callback will be of type G_TYPE_POINTER and contain the pre-parsed
+ * [type@libxml2.Doc]. Do NOT free or modify this document.
  *
  * Return value: %TRUE on success.
  *
@@ -1158,6 +1160,7 @@ subscribe_got_response (GObject *source, GAsyncResult *res, gpointer user_data)
 
         // We don't need the body, it should be empty anyway
         g_input_stream_close (is, NULL, NULL);
+        g_object_unref (is);
 
         /* Remove subscription timeout */
         g_clear_pointer (&priv->subscription_timeout_src, g_source_destroy);
@@ -1215,8 +1218,19 @@ subscribe_got_response (GObject *source, GAsyncResult *res, gpointer user_data)
                         }
 
                         /* We want to resubscribe before the subscription
-                         * expires. */
-                        timeout = g_random_int_range (1, timeout / 2);
+                         * expires. We do that somewhat around the middle of the
+                         * subscription period and introduce some random jitter
+                         * around that, so we do not bombard the services with
+                         * all the re-subscription all at once
+                         */
+                        int jitter =
+                                g_random_int_range (-timeout / 4, timeout / 4);
+                        timeout = timeout / 2 + jitter;
+
+                        g_debug ("Service announced timeout of %s, will "
+                                 "re-subscribe in %d seconds",
+                                 hdr,
+                                 timeout);
 
                         /* Add actual timeout */
                         priv->subscription_timeout_src =
@@ -1592,7 +1606,6 @@ gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
 {
         GUPnPContext *context = NULL;
         SoupSession *session = NULL;
-        GError *internal_error = NULL;
 
         g_return_val_if_fail (GUPNP_IS_SERVICE_PROXY (proxy), NULL);
         g_return_val_if_fail (!action->pending, NULL);
@@ -1606,9 +1619,9 @@ gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
         action->response = soup_session_send_and_read (session,
                                                        action->msg,
                                                        cancellable,
-                                                       &internal_error);
+                                                       &action->error);
 
-        if (internal_error != NULL) {
+        if (action->error != NULL) {
                 goto out;
         }
 
@@ -1618,21 +1631,24 @@ gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
                 if (prepare_action_msg (proxy,
                                         action,
                                         "M-POST",
-                                        &internal_error)) {
+                                        &action->error)) {
                         g_bytes_unref (action->response);
 
                         action->response =
                                 soup_session_send_and_read (session,
                                                             action->msg,
                                                             cancellable,
-                                                            error);
+                                                            &action->error);
                 }
         }
 
+        if (action->error == NULL)
+                gupnp_service_proxy_action_check_response (action);
+
 out:
-        if (internal_error != NULL) {
-                action->error = g_error_copy (internal_error);
-                g_propagate_error (error, internal_error);
+
+        if (action->error != NULL) {
+                g_propagate_error (error, g_error_copy (action->error));
 
                 return NULL;
         }
